@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Parse the UCI Bach choral harmony data set
 
-our $VERSION = '0.0206';
+our $VERSION = '0.0300';
 
 use Moo;
 use strictures 2;
@@ -114,21 +114,35 @@ sub search {
             $results{$id} = $self->data->{$id};
         }
     }
-    elsif ( $args{key} ) {
+
+    if ( $args{key} ) {
+        my @iter = keys %results ? keys %results : keys %{ $self->data };
+
         my @keys = split /\s+/, $args{key};
 
-        for my $id ( keys %{ $self->data } ) {
-            $results{$id} = $self->data->{$id}
-                if any { $_ eq $self->data->{$id}{key} } @keys;
+        for my $id ( @iter ) {
+            if ( $results{$id} ) {
+                delete $results{$id}
+                    unless any { $_ eq $results{$id}{key} } @keys;
+            }
+            else {
+                $results{$id} = $self->data->{$id}
+                    if any { $_ eq $self->data->{$id}{key} } @keys;
+            }
         }
     }
-    elsif ( $args{bass} ) {
-        %results = $self->_search_param( bass => $args{bass} );
+
+    if ( $args{bass} ) {
+        %results = $self->_search_param( bass => $args{bass}, \%results );
     }
-    elsif ( $args{chord} ) {
-        %results = $self->_search_param( chord => $args{chord} );
+
+    if ( $args{chord} ) {
+        %results = $self->_search_param( chord => $args{chord}, \%results );
     }
-    elsif ( $args{notes} ) {
+
+    if ( $args{notes} ) {
+        my @iter = keys %results ? keys %results : keys %{ $self->data };
+
         my $and = $args{notes} =~ /&/ ? 1 : 0;
         my $re  = $and ? qr/\s*&\s*/ : qr/\s+/;
 
@@ -154,9 +168,11 @@ sub search {
             'B'  => 11,
         );
 
-        ID: for my $id ( keys %{ $self->data } ) {
+        ID: for my $id ( @iter ) {
             my %and_notes = ();
 
+            my $match = 0;
+
             for my $event ( @{ $self->data->{$id}{events} } ) {
                 my @bitstring = split //, $event->{notes};
 
@@ -172,9 +188,10 @@ sub search {
                             }
                         }
                         else {
-                            if ( any { defined $index{$_} && $i == $index{$_} } @notes ) {
-                                $results{$id} = $self->data->{$id};
-                                next ID;
+                            for my $note ( sort @notes ) {
+                                if ( defined $index{$note} && $i == $index{$note} ) {
+                                    $match++;
+                                }
                             }
                         }
                     }
@@ -183,19 +200,34 @@ sub search {
                 }
             }
 
-            if ( keys %and_notes ) {
-                my %notes;
-                @notes{@notes} = undef;
+            if ( $and ) {
+                if ( keys %and_notes ) {
+                    my %notes;
+                    @notes{@notes} = undef;
 
-                my $i = 0;
+                    my $i = 0;
 
-                for my $n ( keys %and_notes ) {
-                    $i++
-                        if exists $notes{$n};
+                    for my $n ( keys %and_notes ) {
+                        $i++
+                            if exists $notes{$n};
+                    }
+
+                    if ( $i == scalar keys %notes ) {
+                        $results{$id} = $self->data->{$id};
+                    }
+                    else {
+                        delete $results{$id}
+                            if $results{$id};
+                    }
+                }
+            }
+            else {
+                if ( $results{$id} && $match <= 0 ) {
+                    delete $results{$id};
+                }
+                elsif ( $match > 0 ) {
+                    $results{$id} = $self->data->{$id};
                 }
-
-                $results{$id} = $self->data->{$id}
-                    if $i == scalar keys %notes;
             }
         }
     }
@@ -204,7 +236,9 @@ sub search {
 }
 
 sub _search_param {
-    my ( $self, $name, $param ) = @_;
+    my ( $self, $name, $param, $seen ) = @_;
+
+    my @iter = keys %$seen ? keys %$seen : keys %{ $self->data };
 
     my %results = ();
 
@@ -214,9 +248,11 @@ sub _search_param {
     my %notes = ();
     @notes{ split $re, $param } = undef;
 
-    ID: for my $id ( keys %{ $self->data } ) {
+    ID: for my $id ( @iter ) {
         my %and_notes = ();
 
+        my $match = 0;
+
         for my $event ( @{ $self->data->{$id}{events} } ) {
             if ( $and ) {
                 for my $note ( keys %notes ) {
@@ -227,29 +263,42 @@ sub _search_param {
             }
             else {
                 if ( any { $_ eq $event->{$name} } keys %notes ) {
-                    $results{$id} = $self->data->{$id};
-                    next ID;
+                    $match++;
                 }
             }
         }
 
-        if ( keys %and_notes ) {
-            my $i = 0;
+        if ( $and ) {
+            if ( keys %and_notes ) {
+                my $i = 0;
 
-            for my $n ( keys %and_notes ) {
-                $i++
-                    if exists $notes{$n};
+                for my $n ( keys %and_notes ) {
+                    $i++
+                        if exists $notes{$n};
+                }
+
+                if ( $i == scalar keys %notes ) {
+                    $results{$id} = $self->data->{$id};
+                }
+                else {
+                    delete $results{$id}
+                        if $results{$id};
+                }
+            }
+        }
+        else {
+            if ( $results{$id} && $match <= 0 ) {
+                delete $results{$id};
+            }
+            elsif ( $match > 0 ) {
+                $results{$id} = $self->data->{$id};
             }
-
-            $results{$id} = $self->data->{$id}
-                if $i == scalar keys %notes;
         }
     }
 
     return %results;
 }
 
-
 1;
 
 __END__
@@ -264,7 +313,7 @@ Music::BachChoralHarmony - Parse the UCI Bach choral harmony data set
 
 =head1 VERSION
 
-version 0.0206
+version 0.0300
 
 =head1 SYNOPSIS
 
@@ -291,6 +340,8 @@ version 0.0206
   $songs = $bach->search( chord => 'C_M & D_m' ); # With a C major and a D minor chord
   $songs = $bach->search( notes => 'C E G' );     # With the notes C or E or G
   $songs = $bach->search( notes => 'C & E & G' ); # With C and E and G
+  # Args can be combined too:
+  $songs = $bach->search( key => 'C_M C_m', chord => 'X_m & F_M' );
 
 =head1 DESCRIPTION
 
@@ -374,8 +425,6 @@ The B<bass>, B<chord>, and B<notes> can be searched either as C<or>
 (separating note names with a space character), or as inclusive C<and>
 (separating note names with an C<&> character).
 
-* The items to search for cannot be combined at this time.
-
 =head1 SEE ALSO
 
 The F<eg/*> and F<t/01-methods.t> files in this distribution.
